import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/generated/i18n.dart';
import 'package:provider/provider.dart';

/// Provider封装类
///
/// 方便数据初始化
class ProviderSelectorWidget<T extends ChangeNotifier, S> extends StatefulWidget {
  final ValueWidgetBuilder<S> builder;
  final S Function(BuildContext, T) selector;
  final T model;
  final Widget child;
  final Function(T model) onModelReady;
  final bool autoDispose;
  final ShouldRebuild<S> shoulRebuild;

  ProviderSelectorWidget({
    Key key,
    @required this.builder,
    @required this.model,
    this.selector,
    this.child,
    this.shoulRebuild,
    this.onModelReady,
    this.autoDispose,
  }) : super(key: key);

  _ProviderSelectorWidgetState createState() => _ProviderSelectorWidgetState<T, S>();
}

class _ProviderSelectorWidgetState<T extends ChangeNotifier, S>
    extends State<ProviderSelectorWidget<T, S>> {
  T model;

  @override
  void initState() {
    model = widget.model;
    widget.onModelReady?.call(model);
    super.initState();
  }

  @override
  void dispose() {
    if (widget.autoDispose) model.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T>.value(
      value: model,
      child: Selector<T, S>(
        selector: widget.selector,
        builder: widget.builder,
        shouldRebuild: widget.shoulRebuild,
        child: widget.child,
      ),
    );
  }
}
