import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

import '../request_info.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestListModel<T> extends ViewStateModel{

  /// 页面数据
  List<T> list = [];

  RequestInfo getRequestInfo();


  onCompleted(List<T> data) {}

  void initData({bool retry = false}) async{
    if (retry) {
      setBusy();
    }
    await getData();
  }


  ///获取数据
  Future<List<T>> getData(){
    RequestInfo requestInfo = getRequestInfo();
    return DioUtils.getInstance().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onFinish: (){
          refreshController.resetRefreshState();
        },
        onError: isInit? null : (e){
          handleError(e);
        }
    ).then((res){
      try{
        var data = res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
        list = data;
        onCompleted(data);
        if (isEmpty(data)) {
          setEmpty();
        }else{
          setIdle();
        }
        return data;
      }catch(e){
        handleError(e);
        return null;
      }
    });
  }


  void handleError(e){
    refreshController.finishRefresh(success: false);
    if (e is HttpError) {
      setError(HttpError(HttpError.UNKNOWN, e.toString()), e.toString());
    }else{
      setError(e, e.message, message: e.message);
    }
  }
}