import 'dart:convert';
import 'dart:io';
import 'dart:math';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/utils/win_media.dart';
import 'package:package_info/package_info.dart';
import 'package:path_provider/path_provider.dart';


  ///snackBar
  showSnackBar(BuildContext context, text){
    if(text is String){
      Scaffold.of(context).showSnackBar(SnackBar(content: Text("$text")));
    }else if(text is Widget){
      Scaffold.of(context).showSnackBar(SnackBar(content: text));
    }
  }


  /********************************ObjectUtils******************************/
  /// Returns true if the string is null or 0-length.
  bool isEmptyString(String str) {
    return str == null || str.isEmpty;
  }

  /// Returns true if the list is null or 0-length.
  bool isEmptyList(List list) {
    return list == null || list.isEmpty;
  }

  /// Returns true if there is no key/value pair in the map.
  bool isEmptyMap(Map map) {
    return map == null || map.isEmpty;
  }

  /// Returns true  String or List or Map is empty.
  bool isEmpty(Object object) {
    if (object == null) return true;
    if (object is String && object.isEmpty) {
      return true;
    } else if (object is List && object.isEmpty) {
      return true;
    } else if (object is Map && object.isEmpty) {
      return true;
    }
    return false;
  }

  /// Returns true String or List or Map is not empty.
  bool isNotEmpty(Object object) {
    return !isEmpty(object);
  }

  /// Returns true Two List Is Equal.
  bool twoListIsEqual(List listA, List listB) {
    if (listA == listB) return true;
    if (listA == null || listB == null) return false;
    int length = listA.length;
    if (length != listB.length) return false;
    for (int i = 0; i < length; i++) {
      if (!listA.contains(listB[i])) {
        return false;
      }
    }
    return true;
  }

  /// get length.
  int getLength(Object value) {
    if (value == null) return 0;
    if (value is String) {
      return value.length;
    } else if (value is List) {
      return value.length;
    } else if (value is Map) {
      return value.length;
    } else {
      return 0;
    }
  }


  getImagePath(String name, {String format: 'png'}){
    return 'assets/images/$name.$format';
  }


  String randomStr({int length = 10}){
    String alphabet = 'qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM';
    String left = '';
    for (var i = 0; i < length; i++) {
      left = left + alphabet[Random().nextInt(alphabet.length)];
    }
    return left;
  }




  /********************************File处理******************************/
  ///获取缓存文件夹
  Future<Directory> getCacheDirectory({String path, Future<Directory> directoryPath}) async{
    Directory filepath = isEmpty(directoryPath)? await getTemporaryDirectory() : await directoryPath;
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    Directory directory = Directory(filepath.path + "/" + packageInfo.packageName + (isEmptyString(path)? '' : '/$path'));
    bool exists = await directory.exists();
    if (!exists) {
      await directory.create();
    }
    return directory;
  }



  ///清理文件夹
  void clearDirectory({@required Directory directory, bool isDeleteDirectory = false}){
    if (directory.existsSync()) {
      List<FileSystemEntity> files = directory.listSync();

      if (files.length > 0) {
        files.forEach((file) {
          ///当包含文件夹的时候
          if (file.statSync().type == FileSystemEntityType.directory)
            clearDirectory(directory: Directory(file.path), isDeleteDirectory: true);
          else
            file.deleteSync();
        });
      }
      if (isDeleteDirectory)
        directory.deleteSync();
    }
}

  /********************************JSON处理******************************/
  /// Converts object [value] to a JSON string.
  String encodeObj(Object value) {
    return value == null ? null : json.encode(value);
  }

  /// Converts JSON string [source] to object.
  T getObj<T>(String source, T f(Map v)) {
    if (source == null || source.isEmpty) return null;
    try {
      Map map = json.decode(source);
      return map == null ? null : f(map);
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }

  /// Converts JSON string or JSON map [source] to object.
  T getObject<T>(Object source, T f(Map v)) {
    if (source == null || source.toString().isEmpty) return null;
    try {
      Map map;
      if (source is String) {
        map = json.decode(source);
      } else {
        map = source;
      }
      return map == null ? null : f(map);
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }

  /// Converts JSON string list [source] to object list.
  List<T> getObjList<T>(String source, T f(Map v)) {
    if (source == null || source.isEmpty) return null;
    try {
      List list = json.decode(source);
      return list?.map((value) {
        return f(value);
      })?.toList();
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }

  /// Converts JSON string or JSON map list [source] to object list.
  List<T> getObjectList<T>(Object source, T f(Map v)) {
    if (source == null || source.toString().isEmpty) return null;
    try {
      List list;
      if (source is String) {
        list = json.decode(source);
      } else {
        list = source;
      }
      return list?.map((value) {
        return f(value);
      })?.toList();
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }







