import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/utils/win_media.dart';

///输入框包装类
// ignore: must_be_immutable
class TextFormFieldWidget extends StatefulWidget{

  TextEditingController controller;
  String hintText;
  int maxLenth;
  EdgeInsets margin;
  TextAlign align;
  bool enabled;
  double fontSize;
  double hintSize;
  double suffixIconSize;
  Widget icon;
  IconData clearIcon;
  Icon hidePasswordIcon;
  bool obscureText;
  FocusNode focusNode;
  ValueChanged<String> onFieldSubmitted;
  TextInputAction textInputAction;
  String Function(String text) validator;
  bool showHidePasswordIcon;


  TextFormFieldWidget(this.controller,{this.showHidePasswordIcon = true, this.hintText, this.maxLenth, this.margin,
    this.align = TextAlign.left, this.enabled = true, this.fontSize = 14, this.icon, this. obscureText = false,
    this.onFieldSubmitted, this.textInputAction, this.hintSize = 10, this.clearIcon, this.hidePasswordIcon, this.suffixIconSize = 18, this.focusNode, this.validator});

  @override
  State<StatefulWidget> createState() =>  TextFormFieldWidgetState();

}


class TextFormFieldWidgetState extends State<TextFormFieldWidget>{
  TextEditingController controller;

  /// 默认遮挡密码
  ValueNotifier<bool> obscureNotifier;

  @override
  void initState() {
    controller = widget.controller ?? TextEditingController();
    obscureNotifier = ValueNotifier(widget.obscureText);
    super.initState();
  }

  @override
  void dispose() {
    obscureNotifier.dispose();
    // 默认没有传入controller,需要内部释放
    if (widget.controller == null) {
      controller.dispose();
    }
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 0, horizontal: 8),
      child: ValueListenableBuilder(
        valueListenable: obscureNotifier,
        builder: (context, value, child){
          return TextFormField(
            validator: (value){
              return widget.validator == null? null : widget.validator(value);
            },
            controller: controller,
            textAlign: widget.align,
            focusNode: widget.focusNode,
            obscureText: value,
            style: TextStyle(fontSize: getSp(widget.fontSize)),
            onFieldSubmitted: widget.onFieldSubmitted, ///输入完后的需要操作的事件
            textInputAction: widget.textInputAction,  ///输入完成触发
            decoration: InputDecoration(
                border: InputBorder.none,
                hintText: widget.hintText,
                prefixIcon: isNotEmpty(widget.icon)?Padding(
                  padding: getEdgeInsetsOnly(bottom: 6),
                  child: widget.icon,
                ) : null,
                hintStyle: TextStyle(fontSize: getSp(widget.hintSize)),
                suffixIcon: MySuffixIcon(controller, obscureNotifier, widget.obscureText, widget.hidePasswordIcon, widget.clearIcon, widget.suffixIconSize, widget.showHidePasswordIcon)
            ),
          );
        },
      ),
    );
  }

}



class MySuffixIcon extends StatelessWidget{

  final TextEditingController controller;

  final ValueNotifier<bool> obscureNotifier;

  final bool obscureText;

  final Icon hidePasswordIcon;

  final IconData clearIcon;

  final double suffixIconSize;

  final bool showHidePasswordIcon;

  MySuffixIcon(this.controller, this.obscureNotifier, this.obscureText,
      this.hidePasswordIcon, this.clearIcon, this.suffixIconSize, this.showHidePasswordIcon);

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.end,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        ///密码隐藏控件
        Offstage(
          offstage: !obscureText || !showHidePasswordIcon,
          child: GestureDetector(
            onTap: (){
              obscureNotifier.value = !obscureNotifier.value;
            },
            child: ValueListenableBuilder(
              valueListenable: obscureNotifier,
              builder: (context, value, child) => hidePasswordIcon?? Icon(
                  CupertinoIcons.eye,
                  size: getSp(suffixIconSize),
                  color: value? Theme.of(appContext).hintColor : Theme.of(appContext).accentColor
              ),
            ),
          ),
        ),

        TextFieldClearWidget(controller, suffixIconSize, iconData: clearIcon,),
      ],
    );
  }

}


// ignore: must_be_immutable
class TextFieldClearWidget extends StatefulWidget{

  final TextEditingController controller;

  final IconData iconData;

  final double suffixIconSize;

  TextFieldClearWidget(this.controller, this.suffixIconSize, {this.iconData});


  @override
  State<StatefulWidget> createState() => TextFieldClearState();

}

class TextFieldClearState extends State<TextFieldClearWidget>{

  ///清除按钮监听
  ValueNotifier<bool> clearNotifier;

  @override
  void initState() {
    clearNotifier = ValueNotifier(widget.controller.text.isEmpty);
    ///添加控制器监听,修改ValueNotifier
    widget.controller.addListener(() {
      if (mounted) {
        clearNotifier.value = widget.controller.text.isEmpty;
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder(
      valueListenable: clearNotifier,
      builder: (context, value, child){
        return Offstage(
          offstage: value, ///当offstage为false，显示
          child: GestureDetector(
            onTap: (){
              widget.controller.clear();
            },
            child: Icon(
              widget.iconData??Icons.clear,
              size: getSp(widget.suffixIconSize),
              color: Theme.of(appContext).hintColor,
            ),
          ),
        );
      },
    );
  }

}


