import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_mylibrary/base/provider/request/view_state_request_list_model.dart';
import 'package:flutter_mylibrary/base/provider/request/view_state_request_model.dart';
import 'package:flutter_mylibrary/base/provider/view_state_model.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/utils/win_media.dart';
import 'package:flutter_mylibrary/widget/textformfield_widget.dart';


Widget getNoDataWidget({String error = 'No Data', ViewStateModel viewStateModel, Widget child, GestureDetector onTap}) => Container(
  height: double.infinity,
  alignment: Alignment.center,
  margin: getEdgeInsetsSymmetric(horizontal: defMargin),
  child: child??GestureDetector(
    onTap: onTap??(){
      if (viewStateModel is ViewStateRequestModel) {
        viewStateModel.initData(retry: true);
      }

      if (viewStateModel is ViewStateRequestListModel) {
        viewStateModel.initData(retry: true);
      }

    },
    child: Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        Expanded(
          child: SizedBox(),
          flex: 2,
        ),
        SizedBox(
            width: getWidth(100),
            height: getHeight(100),
            child: Image.asset('assets/images/nodata.png')
        ),
        Text(
          error,
          style: TextStyle(fontSize: getSp(16), color: Colors.grey[400]),
        ),
        Expanded(
          child: SizedBox(),
          flex: 2,
        ),
      ],
    ),
  ),
);


Widget getViewStateWidget(dynamic provider, Function idleWdiget, {Widget busyWidget, Widget emptyDataWidget, Widget errorDataWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
  if(provider.isIdle){
    return idleWdiget();
  }else if(provider.isError){
    if (showErrorWidget) {
      return errorDataWidget??getNoDataWidget(error: provider.viewStateError.message, viewStateModel: provider);
    } else{
      return idleWdiget();
    }
  }else if(provider.isEmpty){
    if (showEmptyWidget) {
      return emptyDataWidget??getNoDataWidget();
    } else{
      return idleWdiget();
    }
  }else if (provider.isBusy) {
    if (showBusyWidget) {
      return busyWidget??pageLoading;
    }else{
      return idleWdiget();
    }
  } else{
    return Container();
  }
}

Widget getRefreshNoDataWidget({ViewStateModel viewStateModel, Widget busyWidget, Widget emptyDataWidget, Widget errorDataWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
  if(viewStateModel.isError && showErrorWidget){
    return errorDataWidget??getNoDataWidget(error: viewStateModel.viewStateError.message, viewStateModel: viewStateModel);
  }else if(viewStateModel.isEmpty && showEmptyWidget){
    return emptyDataWidget??getNoDataWidget();
  }else if (viewStateModel.isBusy && showBusyWidget) {
    return busyWidget??pageLoading;
  }else{
    return null;
  }
}




/// 间隔
class Gaps {
  /// 水平间隔
  static Widget hGap5 = new SizedBox(width: getWidth(5));
  static Widget hGap10 = new SizedBox(width: getWidth(10));
  static Widget hGap15 = new SizedBox(width: getWidth(15));
  static Widget hGapDef = new SizedBox(width: getWidth(defMargin));

  /// 垂直间隔
  static Widget vGap5 = new SizedBox(height: getHeight(5));
  static Widget vGap10 = new SizedBox(height: getHeight(10));
  static Widget vGap15 = new SizedBox(height: getHeight(15));
  static Widget vGapDef = new SizedBox(width: getHeight(defMargin));


  /// 自定义垂直间隔
  static Widget getVertical(double height){
    return SizedBox(
      height: getHeight(height),
    );
  }

  /// 自定义水平间隔
  static Widget getHorizontal(double width){
    return SizedBox(
      width: getHeight(width),
    );
  }
}



/// ******************************公共控件******************************/
Widget getAppBarWidget({bool automaticallyImplyLeading, TextStyle textStyle, Widget titleWidget, double titleTextSize = 16, FontWeight fontWeight, bool centerTitle = true, String title = '', Widget rightWidget, GestureTapCallback onRightWidgetTap, double titleSpacing, PreferredSizeWidget bottom}){
  return AppBar(
    automaticallyImplyLeading: automaticallyImplyLeading??(!isEmptyString(title) && centerTitle),
    titleSpacing: titleSpacing??getWidth(defMargin),
    title: titleWidget??getText(title, fontSize: getSp(titleTextSize), fontWeight: fontWeight, textStyle: textStyle),
    bottom: bottom,
    centerTitle: centerTitle,
    actions: [
      GestureDetector(
        onTap: onRightWidgetTap,
        child: Align(
          alignment: Alignment.center,
          child: Padding(
            padding: getEdgeInsetsOnly(right: 22),
            child: rightWidget,
          ),
        ),
      )
    ],
  );
}



///获取SafeArea的布局
Widget getSafeAreaScaffoldWidget({bool automaticallyImplyLeading, TextStyle textStyle, Widget titleWidget, double titleTextSize = 16, FontWeight fontWeight, bool centerTitle = true, @required Widget body, String title = '', Widget rightWidget, GestureTapCallback onRightWidgetTap, double titleSpacing, EdgeInsets padding}){
  return Scaffold(
    appBar: getAppBarWidget(
      title: title,
      automaticallyImplyLeading: automaticallyImplyLeading,
      titleWidget: titleWidget,
      fontWeight: fontWeight,
      titleTextSize: titleTextSize,
      rightWidget: rightWidget,
      onRightWidgetTap: onRightWidgetTap,
      titleSpacing: titleSpacing,
      centerTitle: centerTitle,
      textStyle: textStyle
    ),
    body: SafeArea(
      child: Padding(
        padding: padding??getEdgeInsetsAll(0),
        child: body,
      ),
    ),
  );
}



Widget getText(String text, {TextStyle textStyle, int maxLines,double fontSize = 14, Color color, EdgeInsets margin, TextDecoration textDecoration, TextAlign textAlign, FontWeight fontWeight, TextOverflow overflow}){
  return Container(
    margin: margin,
    child: Text(
      text??'',
      maxLines: maxLines,
      overflow: overflow,
      textAlign: textAlign,
      style: textStyle??TextStyle(
          decoration: textDecoration,
          fontSize: getSp(fontSize),
          fontWeight: fontWeight,
          color: color??Theme.of(appContext).textTheme.bodyText1.color
      ),
    ),
  );
}





Widget getTextField({@required TextEditingController controller, bool autofocus = false, InputDecoration decoration, String hintText, int maxLength, EdgeInsets margin, TextAlign textAlign = TextAlign.left, bool enabled = true, double fontSize = 14, TextInputType textInputType, int maxLines, List<TextInputFormatter> textInputFormatters, FocusNode focusNode, bool obscureText, TextStyle textStyle}){
  return Container(
    margin: margin,
    child: TextField(
      textInputAction: TextInputAction.done,
      focusNode: focusNode,
      maxLines: isEmpty(obscureText)? maxLines : isEmpty(maxLines)? 1 : maxLines,
      enabled: enabled,
      textAlign: textAlign,
      autofocus: autofocus,
      style: textStyle??TextStyle(fontSize: getSp(fontSize), color: Colors.black),
      maxLength: maxLength,
      keyboardType: textInputType,
      obscureText: obscureText??false,
      inputFormatters: textInputFormatters,
      decoration: decoration??InputDecoration.collapsed(
        border: InputBorder.none,
        hintText: hintText,
      ),
      cursorRadius: Radius.circular(3),
      controller: controller,
    ),
  );
}


Widget getTextWithClearField(TextEditingController textEditingController, double suffixIconSize, {FocusNode focusNode, int maxLines, String hintText, int maxLength, EdgeInsets margin, TextAlign textAlign = TextAlign.left, bool enabled = true, double fontSize = 14, IconData clearIcon, Widget icon}){
  return getTextField(
    margin: margin,
    enabled: enabled,
    textAlign: textAlign,
    fontSize: fontSize,
    maxLength: maxLength,
    maxLines: maxLines,
    focusNode: focusNode,
    decoration: InputDecoration(
      icon: icon,
      border: InputBorder.none,
      hintText: hintText,
      suffixIcon: TextFieldClearWidget(textEditingController, suffixIconSize, iconData: clearIcon,),
    ),
    controller: textEditingController,
  );
}




///间隔listview
Widget getListViewSeparated({@required int itemCount, @required IndexedWidgetBuilder itemBuilder, Axis scrollDirection = Axis.vertical, double divierHeight = 15, Color divierColor, ScrollPhysics physics, EdgeInsets padding,  bool useSliver = false, bool shrinkWrap = false,}){
  return scrollDirection == Axis.vertical? useSliver? SliverList(
    delegate: SliverChildBuilderDelegate(
            (context, index){
          if(index.isOdd){
            return Container(
              height: getHeight(divierHeight),
              color: divierColor,
            );
          }
          return itemBuilder(context, index~/2);
        },
        childCount: itemCount
    ),
  ) : ListView.separated(
    padding: padding??getEdgeInsetsAll(0),
    shrinkWrap: shrinkWrap,
    physics: physics,
    itemCount: itemCount,
    scrollDirection: scrollDirection,
    separatorBuilder: (BuildContext context, int index){
      return Container(
        height: getHeight(divierHeight),
        color: divierColor,
      );
    },
    itemBuilder: itemBuilder
  ) : SingleChildScrollView(
    scrollDirection: scrollDirection,
    child: Row(
      children: List.generate(itemCount, (index) => Container(
        margin: getEdgeInsetsOnly(right: index == itemCount - 1? 0 : divierHeight),
        child: itemBuilder(appContext, index),
      )),
    ),
  );
}




///胶囊控件
Widget getConerContainer({AlignmentGeometry alignment, GestureTapCallback onTap, bool isCircle = false, double borderWidth = 0, Color backgroundColor = Colors.white, double radius = 0, Widget child, EdgeInsets padding, Color borderColor = Colors.white, EdgeInsets margin, BorderRadiusGeometry borderRadius, double height, double width}){
  return GestureDetector(
    onTap: onTap,
    child: Container(
      width: width,
      height: height,
      margin: margin,
      padding: padding,
      alignment: alignment,
      decoration: BoxDecoration(
          shape: isCircle? BoxShape.circle : BoxShape.rectangle,
          color: backgroundColor,
          borderRadius: isCircle? null : borderRadius??BorderRadius.all(Radius.circular(radius)),
          border: borderWidth == 0? null : Border.all(color: borderColor, width: borderWidth)
      ),
      child: child??Container(),
    ),
  );
}


///阴影框
Widget getBoxShadow({AlignmentGeometry alignment, double height, double width, Offset offset, Color boxColor, double blurRadius = 10, BorderRadiusGeometry borderRadiusGeometry, Widget child, EdgeInsetsGeometry padding, BoxBorder border, EdgeInsets margin, double radius = 5}){
  return Container(
      margin: margin,
      padding: padding,
      width: width,
      height: height,
      alignment: alignment,
      decoration: BoxDecoration(
          border: border,
          color: Theme.of(appContext).canvasColor,
          boxShadow: [
            BoxShadow(
              offset: offset??Offset(5.0, 5.0),
              color: boxColor??Color(0xff9cb3b3b3),//阴影颜色
              blurRadius: blurRadius,//阴影大小
            )
          ],
          borderRadius: borderRadiusGeometry??BorderRadius.all(Radius.circular(radius))
      ),
      child: child
  );
}





///圆角图片
Widget getConerNetworkImage({String url, double width, double height, bool isCircle = false, double radius = 5, BoxFit fit = BoxFit.fill}){
  return Container(
    width: isEmpty(width)? null : getWidth(width),
    height: isEmpty(height)? null : getHeight(height),
    decoration: BoxDecoration(
      shape: isCircle? BoxShape.circle : BoxShape.rectangle,
      borderRadius: isCircle? null : BorderRadius.all(Radius.circular(radius)),
      image: DecorationImage(
        image: NetworkImage(url),
        fit: fit,
      ),
    ), // 通过 container 实现圆角
  );
}

///获取以左侧为高度，右侧控件为上下对齐的布局
Widget getFlexRowWidget({Widget leftWidget, Widget rightWidgetTopWidget, Widget rightBottomWidget, double divider}){
  return IntrinsicHeight(
    child: Row(
      children: [
        leftWidget,
        Gaps.getHorizontal(divider),
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Expanded(
                child: rightWidgetTopWidget,
              ),
              rightBottomWidget
            ],
          ),
        )
      ],
    ),
  );
}



